/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.course.nodes.gta.ui;

import org.olat.core.gui.UserRequest;
import org.olat.core.gui.components.Component;
import org.olat.core.gui.components.velocity.VelocityContainer;
import org.olat.core.gui.control.Event;
import org.olat.core.gui.control.WindowControl;
import org.olat.core.gui.control.controller.BasicController;
import org.olat.course.nodes.GTACourseNode;
import org.olat.course.run.userview.UserCourseEnvironment;
import org.olat.modules.ModuleConfiguration;

/**
 * 
 * Initial date: 27.05.2016<br>
 * @author srosse, stephane.rosse@frentix.com, http://www.frentix.com
 *
 */
public class GTACoachManagementController extends BasicController {
	
	private GTACoachAssignementEditController assignmentEditCtrl;
	private GTASampleSolutionsEditController solutionEditCtrl;
	
	public GTACoachManagementController(UserRequest ureq, WindowControl wControl, UserCourseEnvironment userCourseEnv, GTACourseNode gtaNode) {
		super(ureq, wControl);

		ModuleConfiguration config = gtaNode.getModuleConfiguration();
		VelocityContainer mainVC = createVelocityContainer("coach_management");
		
		if(config.getBooleanSafe(GTACourseNode.GTASK_ASSIGNMENT)) {
			assignmentEditCtrl = new GTACoachAssignementEditController(ureq, wControl, gtaNode, userCourseEnv.getCourseEnvironment());
			listenTo(assignmentEditCtrl);
			mainVC.put("tasks", assignmentEditCtrl.getInitialComponent());
		}
		
		if(config.getBooleanSafe(GTACourseNode.GTASK_SAMPLE_SOLUTION)) {
			solutionEditCtrl = new GTASampleSolutionsEditController(ureq, wControl, gtaNode, userCourseEnv.getCourseEnvironment());
			listenTo(solutionEditCtrl);
			mainVC.put("solutions", solutionEditCtrl.getInitialComponent());
		}
		putInitialPanel(mainVC);
	}

	@Override
	protected void doDispose() {
		//
	}

	@Override
	protected void event(UserRequest ureq, Component source, Event event) {
		//
	}

}
